from Products.Scrawl.tests.base import ScrawlTestCase
from Products.Scrawl.config import BLOG_ENTRY_NAME
import string

class TestProductUninstallation(ScrawlTestCase):
    """ Ensure we leave no trace.
    """
    def afterSetUp(self):
        self.portal_types = self.portal.portal_types
        qi = self.portal.portal_quickinstaller
        if qi.isProductInstalled('Scrawl'):
            qi.uninstallProducts(products=['Scrawl',])
    
    def test_types_restored(self):
        qi = self.portal.portal_quickinstaller
        qi.uninstallProducts(products=['Scrawl',])
        # no more Blog Entry
        self.failIf(BLOG_ENTRY_NAME in self.portal_types.objectIds())

    def test_topic_fti_restored(self):
        # no more blog_view
        qi = self.portal.portal_quickinstaller
        qi.uninstallProducts(products=['Scrawl',])
        self.failIf('blog_view' in self.portal_types.Topic.view_methods)

    def test_portal_factory_restored(self):
        self.failIf('Blog Entry' in self.portal.portal_factory.getFactoryTypes(),
                    '"Blog Entry" still in the portal factory, post uninstall.')

    def test_skins_restored(self):
        # no more skins
        qi = self.portal.portal_quickinstaller
        qi.uninstallProducts(products=['Scrawl',])
        skin_names = ('scrawl', 'scrawl_30',)
        portal_skins = self.portal.portal_skins
        all_skins = portal_skins.getSkinSelections()
        for skin in all_skins:
            path = portal_skins.getSkinPath(skin)
            path = map(string.strip, string.split(path,','))
            for skin_name in skin_names:
                self.failIf(skin_name in path,
                    "The skin %s was part of the '%s' skin after uninstall" % (skin_name, skin))


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestProductUninstallation))
    return suite


"""To use, ensure that the contentmigrations product is available
   in your products directory, then add an external method to the 
   portal_skins/custom directory (in the ZMI) such that:
   
   id = migrateQuills
   title = migrateQuills
   module name = Scrawl.migrate_entries
   function name = migrate

   To migrate Quills weblog entries to Scrawl blog entries, simply click
   the 'test' tab. Weblog entries are converted in place to blog entries.
   Cutting and pasting the newly-created blog entries will fix the obvious
   workflow and title weirdness. (You do have a back-up, don't you?)

   For more information on content type migration, see:
   http://plone.org/documentation/tutorial/richdocument/migrations
   
   Migration by Trey Beck
"""


from Products.CMFCore.utils import getToolByName
from StringIO import StringIO
from Products.contentmigration.basemigrator.walker import CatalogWalker
from Products.contentmigration.basemigrator.migrator import CMFItemMigrator

class WeblogEntryMigrator(CMFItemMigrator):

    walkerClass = CatalogWalker
    src_meta_type = 'WeblogEntry'
    src_portal_type = 'WeblogEntry'
    dst_meta_type = 'News Item'
    dst_portal_type = 'Blog Entry'
    map = {'getRawText' : 'setText'}

def migrate(self):
    """Run the migration"""

    out = StringIO()
    print >> out, "Starting migration"

    portal_url = getToolByName(self, 'portal_url')
    portal = portal_url.getPortalObject()
    cat = getToolByName(self, 'portal_catalog')

    migrators = (WeblogEntryMigrator,)

    for migrator in migrators:
        walker = migrator.walkerClass(portal, migrator)
        walker.go(out=out)
        print >> out, walker.getOutput()
    # update the portal_type
    results=cat(portal_type=migrator.dst_portal_type)
    for brain in results:
        ob = brain.getObject()
        ob._setPortalTypeName(migrator.dst_portal_type)
        ob.reindexObject(['portal_type'])
    
    # And let's also update workflows
    #print >> out, "Updating workflow"
    #wf = getToolByName(self, 'portal_workflow')
    #wf.updateRoleMappings()

    # Update Kupu settings
    #print >> out, "Applying Kupu customizations"
    #for type in linkableKupuTypes:
    #    self.addKupuResource('linkable', type)
    #for type in mediaKupuTypes:
    #    self.addKupuResource('mediaobject', type)        
    #for type in collectionKupuTypes:
    #    self.addKupuResource('collection', type)
    
    print >> out, "Migration finished"
    return out.getvalue()



from Products.CMFCore.utils import getToolByName
from Products.Scrawl.config import BLOG_ENTRY_NAME, GLOBALS
from Products.Scrawl import HAS_PLONE30
from Products.Archetypes.Extensions.utils import install_subskin
import string
from StringIO import StringIO

def install(portal):
    out = StringIO()

    # copy the News Item for our blog entry
    portal_types = getToolByName(portal, 'portal_types')
    if BLOG_ENTRY_NAME not in portal_types.objectIds():
        cb_copy_data = portal_types.manage_copyObjects(['News Item'])
        paste_data = portal_types.manage_pasteObjects(cb_copy_data)
        temp_id = paste_data[0]['new_id']
        portal_types.manage_renameObject(temp_id, BLOG_ENTRY_NAME)
        getattr(portal_types, BLOG_ENTRY_NAME).title = BLOG_ENTRY_NAME
        out.write("Duplicated 'News Item' FTI info as '%s'" % BLOG_ENTRY_NAME)
    
    # tweak Blog Entry FTI settings
    blog = getattr(portal_types, BLOG_ENTRY_NAME)
    blog.default_view = 'blogentry_view'
    blog.immediate_view = 'blogentry_view'
    view = 'blogentry_view'
    if view not in blog.view_methods:
        blog._updateProperty('view_methods', blog.view_methods + (view,))
    blog._updateProperty('description', u'A blog entry that will show up in the blog view.')
    blog.allow_discussion = True
    out.write("Tweaked %s FTU settings" % BLOG_ENTRY_NAME)

    # make Blog Entry use portal factory, so we don't have any blog entry skeletons
    factory = getToolByName(portal, 'portal_factory')
    types = factory.getFactoryTypes().keys()
    if 'Blog Entry' not in types:
        types.append('Blog Entry')
        factory.manage_setPortalFactoryTypes(listOfTypeIds=types)
        print >> out, "Added Blog Entry to portal factory"

    # install our skins
    install_subskin(portal, out, GLOBALS)
    skins_tool = getToolByName(portal, "portal_skins")
    if HAS_PLONE30:
        bad_skin = "scrawl"
    else:
        bad_skin = "scrawl_30"

    # Iterate over all existing skins and remove the one we don't want
    skins = skins_tool.getSkinSelections()
    for skin in skins:
        path = skins_tool.getSkinPath(skin)
        path = map(string.strip, string.split(path,','))
        if bad_skin in path:
            path.remove(bad_skin) 
            path = string.join(path, ', ')
            # addSkinSelection will replace existing skins as well.
            skins_tool.addSkinSelection(skin, path)


    # make blog view available to Smart Folders
    view = 'blog_view'
    topic = portal_types.Topic
    if view not in topic.view_methods:
        topic._updateProperty('view_methods', topic.view_methods + (view,))
        print >> out, "Made %s available for topics.\n" % view

    return out.getvalue()

def uninstall(portal, reinstall=False):
    # remove skins
    skins_tool = getToolByName(portal, "portal_skins")
    skin_names = ("scrawl","scrawl_30")
    skins = skins_tool.getSkinSelections()
    for skin in skins:
        path = skins_tool.getSkinPath(skin)
        path = map(string.strip, string.split(path,','))
        path = [s for s in path if s not in skin_names]
        path = string.join(path, ', ')
        skins_tool.addSkinSelection(skin, path)

    # remove blog_view from Topics
    view = 'blog_view'
    portal_types = getToolByName(portal, 'portal_types')
    topic = portal_types.Topic
    if view in topic.view_methods:
        view_methods = [v for v in topic.view_methods if v != view]
        topic._updateProperty('view_methods', view_methods)

    # remove Blog Entry from portal factory
    factory = getToolByName(portal, 'portal_factory')
    types = factory.getFactoryTypes().keys()
    if 'Blog Entry' in types:
        types.remove('Blog Entry')
        factory.manage_setPortalFactoryTypes(listOfTypeIds=types)



from Products.Scrawl.tests.base import ScrawlTestCase
from Products.Scrawl.config import BLOG_ENTRY_NAME
from Products.Scrawl import HAS_PLONE30
import string

class TestProductInstallation(ScrawlTestCase):
    """ Ensure that our product installs properly
    """
    def afterSetUp(self):
        self.portal_types = self.portal.portal_types

    def testBlogEntryInstalled(self):
        self.failUnless(BLOG_ENTRY_NAME in self.portal_types.objectIds())

    def testBlogViewAvailable(self):
        self.failUnless('blog_view' in self.portal_types.Topic.view_methods)

    def testBlogEntryViewAvailable(self):
        """If the default view method isn't listed in the available view methods
        Plone 3.0's type settings protests."""
        be = getattr(self.portal_types, BLOG_ENTRY_NAME)
        self.failUnless('blogentry_view' in be.view_methods)
        
    def testBlogEntryDescription(self):
        """Blog Entry shouldn't show (type) description of the 'News Item'."""
        be = getattr(self.portal_types, BLOG_ENTRY_NAME)
        self.assertEquals(be.description, u'A blog entry that will show up in the blog view.')

    def testPortalFactorySetup(self):
        self.failUnless('Blog Entry' in self.portal.portal_factory.getFactoryTypes(),
                        '"Blog Entry" is not available in the portal factory.')

    def testSkinsTool(self):
        """Test that file system directory views have
           been registered with the skins tool.
        """
        if HAS_PLONE30:
            goodSkin = 'scrawl_30'
            badSkin  = 'scrawl'
        else:
            goodSkin = 'scrawl'
            badSkin  = 'scrawl_30'
        portal_skins = self.portal.portal_skins
        skins = portal_skins.getSkinSelections()
        for skin in skins:
            path = portal_skins.getSkinPath(skin)
            path = map(string.strip, string.split(path,','))

            self.failUnless(goodSkin in path,
                    "The skin %s was part of the '%s' skin" % (goodSkin, skin))
            self.failIf(badSkin in path,
                    "The skin %s was registered with the '%s' skin but shouldn't have been" % (badSkin, skin))

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestProductInstallation))
    return suite


from Testing import ZopeTestCase as ztc
ztc.installProduct('Scrawl')

from Products.PloneTestCase import PloneTestCase
PloneTestCase.setupPloneSite(products=['Scrawl',])

class ScrawlTestCase(PloneTestCase.PloneTestCase):
    """Base class for integration tests.
    """


from Products.CMFCore.DirectoryView import registerDirectory
from Products.Scrawl.config import GLOBALS

registerDirectory('skins', GLOBALS)

# Parts of the installation process depend on the version of Plone.
# This release supports Plone 2.5 and Plone 3.0
try:
    from Products.CMFPlone.migrations import v3_0
except ImportError:
    HAS_PLONE30 = False
else:
    HAS_PLONE30 = True

def initialize(context):
    pass